@@ -164,3 +164,113 @@ GraphicalAlgo <- function(xdata, pk = NULL, Lambda, Sequential_template = NULL,
 
   return(adjacency)
 }
+
+
+#' (Weighted) clustering algorithm
+#'
+#' Runs the (weighted) clustering algorithm specified in the argument
+#' \code{implementation} and returns matrices of variable weights, and the
+#' co-membership structure. This function is not using stability.
+#'
+#' @inheritParams Clustering
+#' @param nc matrix of parameters controlling the number of clusters in the
+#'   underlying algorithm specified in \code{implementation}. If \code{nc} is
+#'   not provided, it is set to \code{seq(1, nrow(xdata))}.
+#' @param Lambda vector of penalty parameters.
+#' @param ... additional parameters passed to the function provided in
+#'   \code{implementation}.
+#'
+#' @return A list with: \item{selected}{matrix of binary selection status. Rows
+#'   correspond to different model parameters. Columns correspond to
+#'   predictors.} \item{weight}{array of model coefficients. Rows correspond to
+#'   different model parameters. Columns correspond to predictors. Indices along
+#'   the third dimension correspond to outcome variable(s).}
+#'   \item{comembership}{array of model coefficients. Rows correspond to
+#'   different model parameters. Columns correspond to predictors. Indices along
+#'   the third dimension correspond to outcome variable(s).}
+#'
+#' @family underlying algorithm functions
+#' @seealso \code{\link{VariableSelection}}
+#'
+#' @examples
+#'
+#' # Simulation of 15 observations belonging to 3 groups
+#' set.seed(1)
+#' simul <- SimulateClustering(
+#'   n = c(5, 5, 5), pk = 100
+#' )
+#'
+#' # Running hierarchical clustering
+#' myclust <- ClusteringAlgo(
+#'   xdata = simul$data, nc = 2:5,
+#'   implementation = HierarchicalClustering
+#' )
+#'
+#' @export
+ClusteringAlgo <- function(xdata,
+                           nc = NULL,
+                           eps = NULL,
+                           Lambda = NULL,
+                           scale = TRUE,
+                           row = TRUE,
+                           implementation = HierarchicalClustering, ...) {
+  # Making sure none of the variables has a null standard deviation
+  mysd <- rep(NA, ncol(xdata))
+  for (j in 1:ncol(xdata)) {
+    mysd[j] <- stats::sd(xdata[, j])
+  }
+  if (any(mysd == 0)) {
+    for (k in which(mysd == 0)) {
+      xdata[, k] <- xdata[, k] + stats::rnorm(n = nrow(xdata), sd = min(mysd[mysd != 0]) / 100)
+    }
+  }
+
+  # Applying user-defined function for variable selection
+  out <- do.call(implementation, args = list(
+    xdata = xdata,
+    nc = nc,
+    eps = eps,
+    Lambda = Lambda,
+    scale = scale,
+    row = row,
+    ...
+  ))
+
+  # Extracting the number of clusters
+  if ("nc" %in% names(out)) {
+    nc <- out$nc
+  } else {
+    if (is.null(Lambda)) {
+      nc <- nc
+    } else {
+      nc <- cbind(rep(nc, nrow(Lambda)))
+    }
+  }
+
+  # Extracting the noise status
+  if ("noise" %in% names(out)) {
+    noise <- out$noise
+  } else {
+    noise <- rep(NA, nrow(xdata))
+  }
+
+  if ("weight" %in% names(out)) {
+    beta_full <- out$weight
+
+    # Setting the beta coefficient to zero for predictors with always the same value (null standard deviation)
+    if (any(mysd == 0)) {
+      if (length(dim(beta_full)) == 2) {
+        beta_full[, which(mysd == 0)] <- 0
+      }
+      if (length(dim(beta_full)) == 3) {
+        beta_full[, which(mysd == 0), ] <- 0
+      }
+    }
+  } else {
+    beta_full <- matrix(1, nrow = length(nc), ncol = ncol(xdata))
+    rownames(beta_full) <- paste0("s", seq(0, nrow(beta_full) - 1))
+    colnames(beta_full) <- colnames(xdata)
+  }
+
+  return(list(comembership = out$comembership, weight = beta_full, nc = nc, noise = noise))
+}
